@@ -54,7 +54,6 @@ qq <-
     function(formula,
              data = parent.frame(),
              aspect = "fill",
-             layout = NULL,
              panel = "panel.qq",
              prepanel = NULL,
              scales = list(),
@@ -64,7 +63,8 @@ qq <-
              xlim,
              ylab,
              ylim,
-             f.value = ppoints,
+             f.value = NULL,
+             drop.unused.levels = TRUE,
              ...,
              subscripts = !is.null(groups),
              subset = TRUE)
@@ -73,17 +73,29 @@ qq <-
     ## dots <- eval(substitute(list(...)), data, parent.frame())
     dots <- list(...)
 
+    groups <- eval(substitute(groups), data, parent.frame())
+    subset <- eval(substitute(subset), data, parent.frame())
+
+    ## Step 1: Evaluate x, y, etc. and do some preprocessing
+    
+    form <-
+        latticeParseFormula(formula, data, subset = subset,
+                            groups = groups, subscripts = TRUE,
+                            drop = drop.unused.levels)
+
+    groups <- form$groups
+
     if (!is.function(panel)) panel <- eval(panel)
     if (!is.function(strip)) strip <- eval(strip)
 
+    if ("subscripts" %in% names(formals(panel))) subscripts <- TRUE
+    if(subscripts) subscr <- form$subscr
+
     prepanel <-
         if (is.function(prepanel)) prepanel 
         else if (is.character(prepanel)) get(prepanel)
         else eval(prepanel)
 
-    ## Step 1: Evaluate x, y, etc. and do some preprocessing
-    
-    form <- latticeParseFormula(formula, data)
     cond <- form$condition
     number.of.cond <- length(cond)
     y <- form$left
@@ -91,18 +103,9 @@ qq <-
     if (number.of.cond == 0) {
         strip <- FALSE
         cond <- list(as.factor(rep(1, length(x))))
-        layout <- c(1,1,1)
         number.of.cond <- 1
     }
 
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
-    if ("subscripts" %in% names(formals(eval(panel)))) subscripts <- TRUE
-    if(subscripts) subscr <- seq(along=x)
-    x <- x[subset, drop = TRUE]
-    y <- y[subset, drop = TRUE]
-    if (subscripts) subscr <- subscr[subset, drop = TRUE]
-    
     ##x <- as.numeric(x)
     y <- as.factorOrShingle(y)
     is.f.y <- is.factor(y)
@@ -122,27 +125,24 @@ qq <-
     ## less complicated components:
 
     foo <- do.call("trellis.skeleton",
-                   c(list(aspect = aspect,
+                   c(list(cond = cond,
+                          aspect = aspect,
                           strip = strip,
                           panel = panel,
                           xlab = xlab,
-                          ylab = ylab), dots))
+                          ylab = ylab,
+                          xlab.default =
+                          if (is.f.y) unique(levels(y))[1]
+                          else paste("y:", as.character(unique(levels(y)[[1]]))),
+
+                          ylab.default =
+                          if (is.f.y) unique(levels(y))[y]
+                          else paste("y:", as.character(unique(levels(y)[[2]])))),
+                     dots))
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
     foo$call <- match.call()
-    foo$fontsize.normal <- 10
-    foo$fontsize.small <- 8
-
-    ## This is for cases like xlab/ylab = list(cex=2)
-    if (is.list(foo$xlab) && !is.characterOrExpression(foo$xlab$label))
-        foo$xlab$label <-
-            if (is.f.y) unique(levels(y))[1]
-            else paste("y:", as.character(unique(levels(y)[[1]])))
-    if (is.list(foo$ylab) && !is.characterOrExpression(foo$ylab$label))
-        foo$ylab$label <-
-            if (is.f.y) unique(levels(y))[y]
-            else paste("y:", as.character(unique(levels(y)[[2]])))
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -192,7 +192,6 @@ qq <-
     
     ## Step 5: Process cond
 
-    cond <- lapply(cond, as.factorOrShingle, subset, drop = TRUE)
     cond.max.level <- unlist(lapply(cond, nlevels))
 
 
@@ -202,81 +201,72 @@ qq <-
     if (!any(!id.na)) stop("nothing to draw")
     ## Nothing simpler ?
 
-    foo$condlevels <- lapply(cond, levels)
-
     ## Step 6: Evaluate layout, panel.args.common and panel.args
 
 
     foo$panel.args.common <- dots
     if (subscripts) foo$panel.args.common$groups <- groups
 
-    layout <- compute.layout(layout, cond.max.level, skip = foo$skip)
-    plots.per.page <- max(layout[1] * layout[2], layout[2])
-    number.of.pages <- layout[3]
-    foo$skip <- rep(foo$skip, length = plots.per.page)
-    foo$layout <- layout
-    nplots <- plots.per.page * number.of.pages
 
-    foo$panel.args <- as.list(1:nplots)
-    cond.current.level <- rep(1,number.of.cond)
-    panel.number <- 1 # this is a counter for panel number
-    for (page.number in 1:number.of.pages)
-        if (!any(cond.max.level-cond.current.level<0))
-            for (plot in 1:plots.per.page) {
+    nplots <- prod(cond.max.level)
+    if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
+    foo$panel.args <- vector(mode = "list", length = nplots)
 
-                if (foo$skip[plot]) foo$panel.args[[panel.number]] <- FALSE
-                else if(!any(cond.max.level-cond.current.level<0)) {
 
-                    id <- !id.na
-                    for(i in 1:number.of.cond)
-                    {
-                        var <- cond[[i]]
-                        id <- id &
-                        if (is.shingle(var))
-                            ((var >=
-                              levels(var)[[cond.current.level[i]]][1])
-                             & (var <=
-                                levels(var)[[cond.current.level[i]]][2]))
-                        else (as.numeric(var) == cond.current.level[i])
-                    }
+    cond.current.level <- rep(1, number.of.cond)
 
-                    if (any(id)) {
 
-                        if (is.f.y) {
-                            tx <- x[id]
-                            ty <- as.numeric(y[id])
-                            x.val <- tx[ty==1]
-                            y.val <- tx[ty==2]
-                        }
-                        else {
-                            tx <- x[id]
-                            ty <- y[id]
-                            ly <- levels(y)
-                            x.val <- tx[ty>=ly[[1]][1] & ty <=ly[[1]][2]]
-                            y.val <- tx[ty>=ly[[2]][1] & ty <=ly[[2]][2]]
-                        }
-                        n <- max(length(x.val), length(y.val))
-                        p  <- f.value(n)
-                        foo$panel.args[[panel.number]] <-
-                            list(x = quantile(x = x.val, probs = p),
-                                 y = quantile(x = y.val, probs = p))
+    for (panel.number in seq(length = nplots))
+    {
 
-                    }
-                    else
-                        foo$panel.args[[panel.number]] <-
-                            list(x = numeric(0), y = numeric(0))
+        id <- !id.na
+        for(i in 1:number.of.cond)
+        {
+            var <- cond[[i]]
+            id <- id &
+            if (is.shingle(var))
+                ((var >=
+                  levels(var)[[cond.current.level[i]]][1])
+                 & (var <=
+                    levels(var)[[cond.current.level[i]]][2]))
+            else (as.numeric(var) == cond.current.level[i])
+        }
 
-                    if (subscripts)
-                        foo$panel.args[[panel.number]]$subscripts <-
-                            subscr[id]
+        if (any(id)) {
 
-                    cond.current.level <-
-                        cupdate(cond.current.level,
-                                cond.max.level)
-                }
-
-                panel.number <- panel.number + 1
+            if (is.f.y) {
+                tx <- x[id]
+                ty <- as.numeric(y[id])
+                x.val <- tx[ty==1]
+                y.val <- tx[ty==2]
             }
+            else {
+                tx <- x[id]
+                ty <- y[id]
+                ly <- levels(y)
+                x.val <- tx[ty>=ly[[1]][1] & ty <=ly[[1]][2]]
+                y.val <- tx[ty>=ly[[2]][1] & ty <=ly[[2]][2]]
+            }
+            n <- max(length(x.val), length(y.val))
+            ## changed from S-PLUS, where f.value = ppoints is default
+            p  <- if (is.null(f.value)) ppoints(n, a = 1) else f.value(n)
+            foo$panel.args[[panel.number]] <-
+                list(x = quantile(x = x.val, probs = p),
+                     y = quantile(x = y.val, probs = p))
+
+        }
+        else
+            foo$panel.args[[panel.number]] <-
+                list(x = numeric(0), y = numeric(0))
+
+        if (subscripts)
+            foo$panel.args[[panel.number]]$subscripts <-
+                subscr[id]
+
+        cond.current.level <-
+            cupdate(cond.current.level,
+                    cond.max.level)
+    }
 
     foo <- c(foo,
              limits.and.aspect(prepanel.default.qq,
